@@ -22,7 +22,6 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		data.sort <- data.frame(data[,2], data[,2],row.names=data[,1])
 	}
 	data.sort<-data.sort[phy$tip.label,]
-	
 	#Some initial values for use later
 	k=2
 	obj <- NULL
@@ -34,9 +33,13 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		if(is.null(rate.mat)){
 			rate<-rate.mat.maker(hrm=TRUE,rate.cat=rate.cat)
 			rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
+			drop.states = NULL
 		}
 		else{
-			rate<-rate.mat
+			rate <- rate.mat
+            col.sums <- which(colSums(rate.mat, na.rm=TRUE) == 0)
+            row.sums <- which(rowSums(rate.mat, na.rm=TRUE) == 0)
+            drop.states <- col.sums[which(col.sums == row.sums)]
 			rate[is.na(rate)]<-max(rate,na.rm=TRUE)+1
 		}
 		#Makes a matrix of tip states and empty cells corresponding 
@@ -108,6 +111,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		tranQ <- matrix(0, k*rate.cat, k*rate.cat)
 	}
 	if(hrm==FALSE){
+		drop.states = NULL
 		#Imported from Jeffs rayDISC -- will clean up later, but for now, it works fine:
 		if(ntraits==1){
 			k <- 1
@@ -241,9 +245,12 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 		tranQ <- matrix(0, nl^k, nl^k)
 	}
 	
-    p[p==0] = exp(-21)
+    if(length(drop.states > 0)){
+        liks[,drop.states] <- 0
+    }
+
+	p[p==0] = exp(-21)
 	Q[] <- c(p, 0)[rate]
-	diag(Q) <- -rowSums(Q)
 	phy <- reorder(phy, "pruningwise")
 	TIPS <- 1:nb.tip
 	anc <- unique(phy$edge[,1])
@@ -475,6 +482,17 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 			comp[focal] <- sum(v)
 			liks.final[focal, ] <- v/comp[focal]
 		}
+		#Now get the states for the tips (will do, not available for general use):
+		for (i in seq(from = 1, length.out = length(TIPS))) { 
+			#the ancestral node at row i is called focal
+			focal <- TIPS[i]
+			focalRows <- which(phy$edge[,2]==focal)
+			#Now you are assessing the change along the branch subtending the focal by multiplying the probability of 
+			#everything at and above focal by the probability of the mother and all the sisters given time t:
+			v <- liks.down[focal,]*expm(tranQ * phy$edge.length[focalRows], method=c("Ward77")) %*% liks.up[focal,]
+			comp[focal] <- sum(v)
+			liks.final[focal, ] <- v/comp[focal]
+		}		
 		#Just add in the marginal at the root calculated on the original downpass or if supplied by the user:
 		liks.final[root,] <- liks.down[root,] * root.p
 		root.final <- liks.down[root,] * root.p
@@ -489,6 +507,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
 	
 	if(method=="scaled"){
 		comp<-matrix(0,nb.tip + nb.node,ncol(liks))
+        root <- nb.tip + 1L
 		#The same algorithm as in the main function. See comments in either corHMM.R, corDISC.R, or rayDISC.R for details:
 		for (i  in seq(from = 1, length.out = nb.node)) {
 			#the ancestral node at row i is called focal
@@ -516,29 +535,28 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), hrm=
             k.rates <- 1/length(which(!is.na(equil.root)))
             flat.root[!is.na(flat.root)] = k.rates
             flat.root[is.na(flat.root)] = 0
-            liks[root, ] <- flat.root * liks[root, ]
-            liks[root, ] <- liks[root,] / sum(liks[root, ])
-        }
-        else{
+            liks[root,] <- flat.root * liks[root,]
+            liks[root,] <- liks[root,] / sum(liks[root,])
+        }else{
             if(is.character(root.p)){
                 # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
                 if(root.p == "yang"){
                     Q.tmp <- Q
                     diag(Q.tmp) = 0
                     root.p = colSums(Q.tmp) / sum(Q.tmp)
-                    liks[root, ] <- root.p * liks[root, ]
-                    liks[root, ] <- liks[root,] / sum(liks[root, ])
+                    liks[root,] <- root.p * liks[root, ]
+                    liks[root,] <- liks[root,] / sum(liks[root,])
                 }else{
                     # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
                     root.p = liks[root,] / sum(liks[root,])
-                    liks[root, ] <- root.p * liks[root, ]
-                    liks[root, ] <- liks[root,] / sum(liks[root, ])
+                    liks[root,] <- root.p * liks[root,]
+                    liks[root,] <- liks[root,] / sum(liks[root,])
                 }
             }
             # root.p!==NULL will fix root probabilities based on user supplied vector:
             else{
-                liks[root, ] <- root.p * liks[root, ]
-                liks[root, ] <- liks[root,] / sum(liks[root, ])
+                liks[root,] <- root.p * liks[root,]
+                liks[root,] <- liks[root,] / sum(liks[root,])
             }
         }
 		#Reports the probabilities for all internal nodes as well as tips:
